import {select} from 'd3-selection';
import {enableFetchMocks, disableFetchMocks} from 'jest-fetch-mock';
import mockConsole from 'jest-mock-console';

import config from 'ui/config';
import * as utils from 'ui/utils';

import {getSelectedTimeSpan, getGraphBrushOffset} from 'ml/selectors/hydrograph-state-selector';

import {configureStore} from 'ml/store';
import * as hydrographData from 'ml/store/hydrograph-data';
import {setSelectedParameterCode, setSelectedTimeSpan} from 'ml/store/hydrograph-state';

import {drawTimeSpanShortcutButtons} from './time-span-shortcuts';
import * as dataIndicator from './data-indicator';

const TEST_STATE = {
    hydrographState: {
        selectedTimeSpan: 'P7D',
        selectedParameterCode: '00060',
        graphBrushOffset: {
            start: 0,
            end: 10000000000
        }
    }
};

const TEST_IV_PERIOD_OF_RECORD = {
    '00010': {
        begin_date: '2020-01-01',
        end_date: '2021-10-03'
    },
    '00060': {
        begin_date: '2019-01-01',
        end_date: '2021-10-01'
    },
    '72019': {
        begin_date: '2019-11-11',
        end_date: '2021-11-03'
    }
};

const TEST_GW_PERIOD_OF_RECORD = {
    '72019': {
        begin_date: '2020-01-01',
        end_date: '2021-11-04'
    },
    '62611': {
        begin_date: '2019-01-15',
        end_date: '2021-12-04'
    }
};

describe('monitoring-location/components/hydrograph/time-span-shortcuts', () => {
    utils.mediaQuery = jest.fn().mockReturnValue(true);

    let div;
    let store;
    let retrieveSpy;
    let showDataIndicatorSpy;
    config.locationTimeZone = 'America/Chicago';
    config.ivPeriodOfRecord = TEST_IV_PERIOD_OF_RECORD;
    config.gwPeriodOfRecord = TEST_GW_PERIOD_OF_RECORD;

    let restoreConsole;
    beforeAll(() => {
        enableFetchMocks();
        restoreConsole = mockConsole();
    });

    afterAll(() => {
        disableFetchMocks();
        restoreConsole();
    });

    beforeEach(() => {
        div = select('body').append('div');
        store = configureStore(TEST_STATE);
        retrieveSpy = jest.spyOn(hydrographData, 'retrieveHydrographData');
        showDataIndicatorSpy = jest.spyOn(dataIndicator, 'showDataIndicators');
    });

    afterEach(() => {
        div.remove();
    });

    describe('drawShortcutDaysBeforeButtons', () => {
        it('Expects to render the shortcut radio buttons and check the 7 day radio button', () => {
            drawTimeSpanShortcutButtons(div, store, '11112222');
            const radio7Day = div.select('#P7D-input');
            const radio30Day = div.select('#P30D-input');
            const radio1Year = div.select('#P365D-input');

            expect(div.select('.iv-button-container').attr('hidden')).toBeFalsy();
            expect(div.select('.gw-button-container').attr('hidden')).toBeTruthy();

            expect(radio7Day.attr('type')).toBe('radio');
            expect(radio7Day.property('checked')).toBe(true);

            expect(radio30Day.attr('type')).toBe('radio');
            expect(radio30Day.property('checked')).toBe(false);

            expect(radio1Year.attr('type')).toBe('radio');
            expect(radio1Year.property('checked')).toBe(false);
        });

        it('Expects that if the selectedTimeSpan is changed to a days before that is not a shortcut, they are all unset', () => {
            drawTimeSpanShortcutButtons(div, store, '11112222');
            store.dispatch(setSelectedTimeSpan('P45D'));
            return new Promise(resolve => {
                window.requestAnimationFrame(() => {
                    const radio7Day = div.select('#P7D-input');
                    const radio30Day = div.select('#P30D-input');
                    const radio1Year = div.select('#P365D-input');
                    expect(radio7Day.property('checked')).toBe(false);
                    expect(radio30Day.property('checked')).toBe(false);
                    expect(radio1Year.property('checked')).toBe(false);
                resolve();
                });
            });
        });

        it('Expects that if the selectedTimeSpan is changed to a date range, the shortcut radio buttons are not checked', () => {
            drawTimeSpanShortcutButtons(div, store, '11112222');
            store.dispatch(setSelectedTimeSpan({start: '2020-01-03', end: '2020-05-01'}));
            return new Promise(resolve => {
                window.requestAnimationFrame(() => {
                    const radio7Day = div.select('#P7D-input');
                    const radio30Day = div.select('#P30D-input');
                    const radio1Year = div.select('#P365D-input');
                    expect(radio7Day.property('checked')).toBe(false);
                    expect(radio30Day.property('checked')).toBe(false);
                    expect(radio1Year.property('checked')).toBe(false);
                    resolve();
                });
            });
        });

        it('Expects that clicking a radio button updates the selectedTimeSpan and retrieve the data', () => {
            drawTimeSpanShortcutButtons(div, store, '11112222', 'USGS');
            const radio30Day = div.select('#P30D-input');
            radio30Day.property('checked', true);
            radio30Day.dispatch('click');

            let state = store.getState();
            expect(retrieveSpy).toHaveBeenCalledWith('11112222', 'USGS', {
                parameterCode: '00060',
                period: 'P30D',
                startTime: null,
                endTime: null,
                loadCompare: false,
                loadMedian: false
            });
            expect(showDataIndicatorSpy).toHaveBeenCalled();
            expect(getSelectedTimeSpan(state)).toEqual('P30D');
            expect(getGraphBrushOffset(state)).toBeNull();
        });

        it('Expects that if the parameter code is changed to a gw only code the radio buttons will be different', () => {
            drawTimeSpanShortcutButtons(div, store, '11112222', 'USGS');
            store.dispatch(setSelectedParameterCode('62611'));
            return new Promise(resolve => {
                window.requestAnimationFrame(() => {
                    expect(div.select('.iv-button-container').attr('hidden')).toBeTruthy();
                    expect(div.select('.gw-button-container').attr('hidden')).toBeFalsy();

                    resolve();
                });
            });
        });
    });
});